<?php

/*

	FASDIM: Fast And Simple DeIdentification Method
	Copyright (C) 2013 Emmanuel Chazard http://emmanuel.chazard.org
	Check for updates on http://fasdim.org
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 3 of the License.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.



*/

// check the argument of the script (number of the step)
if( $argc != 2) {
	trigger_error('You must precise the number of the step as argument (1, 3 or 6)', E_USER_ERROR) ;	
} elseif (!in_array( $argv[1], array(1, 3, 6))) {
	trigger_error('The number of the step must equal 1, 3 or 6. '.$argv[1].' given in the command line.', E_USER_ERROR) ;
} else {
	$step = $argv[1] ;
}

// only for a pretty display of the needed parameters. Not mandatory.
if( $step==1) {
	$liste_param=array('doc_nonano', 'txt_nonano', 'txt_preano', 'txt_ano',	'database_dsn', 'database_user', 'database_password', 'table_docnonano2txtnonano') ;
} elseif( $step==3) {
	$liste_param=array('txt_nonano', 'txt_preano',	'database_dsn', 'database_user', 'database_password', 'table_txtnonano2txtpreano') ;
} elseif( $step==6) {
	$liste_param=array('txt_preano', 'txt_ano', 'database_dsn', 'database_user', 'database_password', 'table_txtpreano2txtano') ;
} else {
	trigger_error('The number of the step must equal 1, 3 or 6. '.$step.' given in the command line.', E_USER_ERROR) ;
}


/* ======================= INCLUSION ============================= */

require_once('./scripts/fasdim_include.inc') ;


$rep_courriers_doc =  $configuration->get('doc_nonano') ;
$rep_courriers_txt_nonano =  $configuration->get('txt_nonano') ;
$rep_courriers_txt_preano =  $configuration->get('txt_preano') ;
$rep_courriers_txt_ano =  $configuration->get('txt_ano') ;

/* ======================= The behaviour will depend on the step ============================= */

switch ($step) {
	case 1:
		$announce = "\nSTEP 01\nThis program lists your DOC files (doc_nonano).\nA MySQL table will be created. A FASDIM database must first exist.\nThe DOC files must already exist in the corresponding directory.\nDOCX files are not taken into account.\n\n" ;
		$dirs_to_control_strictly = array($rep_courriers_doc) ;
		$dirs_to_control_or_create = array($rep_courriers_txt_nonano, $rep_courriers_txt_preano, $rep_courriers_txt_ano) ;
		$dir_to_list = $rep_courriers_doc ;
		$extensions = array('doc') ;
		$table_name = $configuration->get('table_docnonano2txtnonano') ;
		break;
	case 3:
		$announce = "\nSTEP 03\nThis program lists the TXT files candidate for de-identification (txt_nonano).\n\n" ;
		$dirs_to_control_strictly = array($rep_courriers_txt_nonano) ;
		$dirs_to_control_or_create = array($rep_courriers_txt_preano, $rep_courriers_txt_ano) ;
		$dir_to_list = $rep_courriers_txt_nonano ;
		$extensions = array('txt') ;
		$table_name = $configuration->get('table_txtnonano2txtpreano') ;
		break;
	case 6:
		$announce = "\nSTEP 06\nThis program lists the TXT files for final de-identification (txt_preano).\n\n" ;
		$dirs_to_control_strictly = array($rep_courriers_txt_preano) ;
		$dirs_to_control_or_create = array($rep_courriers_txt_ano) ;
		$dir_to_list = $rep_courriers_txt_preano ;
		$extensions = array('txt') ;
		$table_name = $configuration->get('table_txtpreano2txtano') ;
		break;
	default:
		trigger_error('The number of the step must equal 1, 3 or 6. '.$step.' given in the command line.', E_USER_ERROR) ;
}


/* ======================= Main ============================= */

print($announce) ;


// we control whether the directories exist
foreach( $dirs_to_control_strictly as $a_dir) {
	if( !is_dir($a_dir)) {
		trigger_error( "The directory $a_dir must exist and contain all the files", E_USER_ERROR) ;
	}
}
foreach( $dirs_to_control_or_create as $a_dir) {
	if( !is_dir($a_dir)) {
		mkdir($a_dir) ;
		trigger_error( "Directory creation: $a_dir", E_USER_NOTICE) ;
	}
}

// We list the files to register into the database
print "\nListing of the files" ;
$bdd->exec("CREATE TABLE $table_name ( nom varchar(255) not null, statut char(1) not null, primary key (nom) )  ;");
$stt = $bdd->prepare("INSERT INTO $table_name (nom, statut) VALUES (:nom, :statut) ;") ;

$file_list = glob( $dir_to_list ."/*.*" ) ;
$files_number = count($file_list) ;

$progression_bar = new batch_progression_bar(0, $files_number ) ;
$progression_bar->draw() ;
$i=0 ;

$nb_indexed=0 ;
$nb_ignored=0 ;
$extensions_found = array() ;

foreach( $file_list as $a_file) {
	$a_file = basename($a_file) ;
	$select = TRUE ;
	$tab = explode('.', $a_file) ;
	$ext = $tab[(count($tab)-1)] ;
	if( !isset($extensions_found[$ext])) {
		$extensions_found[$ext]=0 ;
	}
	$extensions_found[$ext]++ ;
	if( !in_array( strtolower($ext), $extensions) ) {
		$select=FALSE ;
	}
	if($select) {
		$stt->execute(array(":nom"=>$a_file, ":statut"=>0)) ;
		$nb_indexed++ ;
	} else {
		$nb_ignored++ ;
	}
	$i++ ;
	$progression_bar->forward($i) ;
}

print "\nIndexed files: $nb_indexed" ;
print "\nIgnored files: $nb_ignored" ;
print "\nFile extensions:\n" ;
print_r($extensions_found) ;


print("\n\n\nSTEP $step FINISHED!\nCheck the error log file if necessary.\n\n\n") ;

?>