<?php

/*

	FASDIM: Fast And Simple DeIdentification Method
	Copyright (C) 2013 Emmanuel Chazard http://emmanuel.chazard.org
	Check for updates on http://fasdim.org
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 3 of the License.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.


*/

print("
STEP 04
This program prepares TXT files for the de-identification. It is language-dependent:
- typographic simplification
- suppression of salutation patterns
- suppression of date patterns
- suppression of patient names (if known)
If necessary, several instances of the program can be launched in the same time.
") ;

$liste_param=array( 'txt_nonano', 'txt_preano',
	'database_dsn', 'database_user', 'database_password',
	'table_txtnonano2txtpreano',
	'file_reports', 'file_patients_names'
) ;


require_once('./scripts/fasdim_include.inc') ;


/* ======================= Parameters ============================= */

$dir_in =  $configuration->get('txt_nonano') ;
$dir_out =  $configuration->get('txt_preano') ;
$file_with_names = $configuration->get('file_patients_names') ;
$file_with_reports = $configuration->get('file_reports') ;
$table_name = $configuration->get('table_txtnonano2txtpreano') ;

/* ======================= Functions ============================= */


function clean_text_fr($content) {
	$content = trim($content) ;
	
	// The apostrophe is strangely handled by WVWare
	$content = str_replace( chr(25), ' ', $content);
	
	// string to lower
	$content = strtolower( $content ) ;

	// handling of accents and special characters
	$content = str_replace( array('', '', ''), 'a', $content);
	$content = str_replace( array('', '', '', ''), 'e', $content);
	$content = str_replace( array( '', ''), 'i', $content);
	$content = str_replace( array( '', ''), 'o', $content);
	$content = str_replace( array( '', '', ''), 'u', $content);
	$content = str_replace( '', 'y', $content);
	$content = str_replace('', 'oe', $content) ;
	$content = str_replace('', 'ae', $content) ;
	$content = str_replace('', 'c', $content) ;

	// Salutation patterns with vorname and family name
	$content = preg_replace(array(
		'|\bmonsieur\s+\w+\s+\w+\b|',
		'|\bm\.\s+\w+\s+\w+\b|',
		'|\bmr\.\s+\w+\s+\w+\b|',
		'|\bmr\s+\w+\s+\w+\b|',
		'|\bmadame\s+\w+\s+\w+\b|',
		'|\bmme\.\s+\w+\s+\w+\b|',
		'|\bmme\s+\w+\s+\w+\b|',
		'|\bmademoiselle\s+\w+\s+\w+\b|',
		'|\bmelle\.\s+\w+\s+\w+\b|',
		'|\bmelle\s+\w+\s+\w+\b|',
		'|\bdocteur\s+\w+\s+\w+\b|',
		'|\bdr\.\s+\w+\s+\w+\b|',
		'|\bdr\s+\w+\s+\w+\b|',
		'|\bprofesseur\s+\w+\s+\w+\b|',
		'|\bpr\.\s+\w+\s+\w+\b|',
		'|\bpr\s+\w+\s+\w+\b|',
	), '@ @ @', $content) ;

	// Salutation patterns with first letter of the vorname, a dot, and the family name
	$content = preg_replace(array(
		'|\bmonsieur\s+\w\.\s?\w+\b|',
		'|\bm\.\s+\w\.\s?\w+\b|',
		'|\bmr\.\s+\w\.\s?\w+\b|',
		'|\bmr\s+\w\.\s?\w+\b|',
		'|\bmadame\s+\w\.\s?\w+\b|',
		'|\bmme\.\s+\w\.\s?\w+\b|',
		'|\bmme\s+\w\.\s?\w+\b|',
		'|\bmademoiselle\s+\w\.\s?\w+\b|',
		'|\bmelle\.\s+\w\.\s?\w+\b|',
		'|\bmelle\s+\w\.\s?\w+\b|',
		'|\bdocteur\s+\w\.\s?\w+\b|',
		'|\bdr\.\s+\w\.\s+\w?\b|',
		'|\bdr\s+\w\.\s+\w?\b|',
		'|\bprofesseur\s+\w\.\s?\w+\b|',
		'|\bpr\.\s+\w\.\s?\w+\b|',
		'|\bpr\s+\w\.\s?\w+\b|',
	), '@ @ @', $content) ;

	// Salutation patterns with only the family name
	$content = preg_replace(array(
		'|\bmonsieur\s+\w+\b|',
		'|\bm\.\s+\w+\b|',
		'|\bmr\.\s+\w+\b|',
		'|\bmr\s+\w+\b|',
		'|\bmadame\s+\w+\b|',
		'|\bmme\.\s+\w+\b|',
		'|\bmme\s+\w+\b|',
		'|\bmademoiselle\s+\w+\b|',
		'|\bmelle\.\s+\w+\b|',
		'|\bmelle\s+\w+\b|',
		'|\bdocteur\s+\w+\b|',
		'|\bdr\.\s+\w+\b|',
		'|\bdr\s+\w+\b|',
		'|\bprofesseur\s+\w+\b|',
		'|\bpr\.\s+\w+\b|',
		'|\bpr\s+\w+\b|',
	), '@ @', $content) ;

	// Some dates and related
	$content = preg_replace(array(
		'|\blundi\b|',
		'|\bmardi\b|',
		'|\bmercredi\b|',
		'|\bjeudi\b|',
		'|\bvendredi\b|',
		'|\bsamedi\b|',
		'|\bdimanche\b|',
		'|\bjanvier\b|',
		'|\bfevrier\b|',
		'|\bmars\b|',
		'|\bavril\b|',
		'|\bmai\b|',
		'|\bjuin\b|',
		'|\bjuillet\b|',
		'|\baout\b|',
		'|\bseptembre\b|',
		'|\boctobre\b|',
		'|\bnovembre\b|',
		'|\bdecembre\b|'
	), '@', $content) ;

	return $content ;
}


function split_name_fr($string) {
	$final = array() ;
	$words1 = explode('-', $string) ;
	$words2 = preg_split("/[\s\-\']+/", trim($string));
	$list = array_merge( array($string), $words1, $words2 ) ;
	foreach( $list as $a_word ) {
		$a_word = trim($a_word) ;
		if(strlen($a_word)>2) {
			$final[] = $a_word ;
		}
	}
	return array_unique($final) ;
}


/*  =========== Patients names loading =========== */

if( !is_file( $file_with_names ) or !is_file( $file_with_reports )) {

	trigger_error("Patient names won't be suppressed, because the related files are not available.", E_USER_WARNING) ;
	$patient_names_deletion = FALSE ;

} else {

	$patient_names_deletion = TRUE ;

	// find the ID of the patients from the file name
	$reports_memory = array() ;

	$fp = fopen( $file_with_reports, 'r' ) ;
	if( !$fp ) {
		trigger_error( "The file $file_with_reports cannot be found", E_USER_ERROR ) ;
	}
	while( !feof( $fp ) ) {
		$line = fgets( $fp ) ;
		$cells = explode("\t", $line) ;
		if( count($cells)>3) {
			$id_patient = ltrim( trim($cells[2]), ' 0') ; ;
			$id_letter = $cells[4] ;
			$id_letter = str_replace('.doc', '.txt', trim($id_letter) ) ;  /* ... as the files have been renamed previously... */
			$reports_memory[ $id_letter ] = $id_patient ;
		}
	}
	fclose( $fp ) ;

	// For a given patient, the names to delete
	$words_memory = array() ;

	$file = $file_with_names ;
	$fp = fopen( $file, 'r' ) ;
	if( !$fp ) {
		trigger_error( "File not found $file", E_USER_ERROR ) ;
	}
	while( ! feof( $fp ) ) {
		$line = fgets( $fp ) ;
		$cells = explode("\t", $line) ;
		if( count($cells)>4) {
			$id_patient = ltrim( $cells[1], ' 0') ; ;
			$noms = array_unique(array_merge(
				split_name_fr($cells[2]),
				split_name_fr($cells[3]),
				split_name_fr($cells[4])
					
			)) ;
			$words_memory[ $id_patient ] = $noms ;
		}
	}
	fclose( $fp ) ;
}






/*  =========== MAIN =========== */



print( $bdd->get_string_lock_status($table_name) );
$i=0 ;
print "\n\n\n$i records" ;


while( $file = $bdd->lock_file($table_name) ) {
	$a_file_in = $dir_in.'/'. $file ;
	$a_file_out = $dir_out.'/'. $file ;
	$content = file_get_contents( $a_file_in, FILE_TEXT ) ;
	// typographic purification, and some patterns deletion
	$content = clean_text_fr($content) ;

	// If we have them, we drop the patients names
	if( $patient_names_deletion  ) {
		$words = array() ;
		if( isset( $reports_memory[$file] ) ) {
			$id_patient = $reports_memory[$file] ;
			if( isset( $words_memory[ $id_patient ] ) and count($words_memory[ $id_patient ])>0) {
				$words = $words_memory[ $id_patient ] ;
			} else {
				trigger_error("The record $file couldn't be fully de-identified (undefined words)", E_USER_NOTICE) ;
			}
		} else {
			trigger_error("The record $file couldn't be fully de-identified (unknown patient)", E_USER_NOTICE) ;
		}
		foreach( $words as $a_word ) {
			if( !strstr($a_word, '.' ) AND !strstr($a_word, '+' ) AND !strstr($a_word, '*' ) AND !strstr($a_word, '?') ) {
				$content = preg_replace("|\b$a_word\b|iU", '@', $content )  ;
			}
		}
	}

	// We save the new file
	file_put_contents( $a_file_out, $content, FILE_TEXT ) ;
	$bdd->unlock_file($table_name, $file) ;
	$i++ ;
	print "\r$i reports" ;
}


print( "\n\n". $bdd->get_string_lock_status($table_name) . "\n\n" );



?>



STEP 04 DONE
Please check the error log file if necessary.


