<?php


/*

	FASDIM: Fast And Simple DeIdentification Method
	Copyright (C) 2013 Emmanuel Chazard http://emmanuel.chazard.org
	Check for updates on http://fasdim.org
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 3 of the License.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.


*/

print("
STEP 07
This program proceeds the final de-identification of the prepared TXT files.
Several instances of the program can be launched in the same time.
") ;

$liste_param=array('txt_preano', 'txt_ano',
	'database_dsn', 'database_user', 'database_password',
	'table_txtpreano2txtano',
	'tokens_to_protect', 'numbers_to_protect'
) ;


require_once('./scripts/fasdim_include.inc') ;


/* ======================= Parameters ============================= */

$file_in_words_dictionnary = $configuration->get('tokens_to_protect') ;
$file_in_number_dictionnary = $configuration->get('numbers_to_protect') ;
$dir_in =  $configuration->get('txt_preano') ;
$dir_out =  $configuration->get('txt_ano') ;
$table_name = $configuration->get('table_txtpreano2txtano') ;




/* ======================= Main ============================= */

// ================ we load the dictionnaries



// we load the dictionnary of authorized words
$words_dictionnary = array() ;
$fp = fopen( $file_in_words_dictionnary, 'r' ) ;
if( !$fp ) {
	trigger_error( "File not found $file_in_words_dictionnary", E_USER_ERROR ) ;
}
while( !feof( $fp ) ) {
	$line = fgets( $fp ) ;
	$cells = explode("\t", $line) ;
	$mot = trim($cells[0]) ;
	if( strlen($mot)>0) {
		$words_dictionnary[] = $mot ;
	}
}
fclose( $fp ) ;

print "\nThe word dictionnary contains ". count($words_dictionnary) ." words.\n" ;


// we load the dictionnary of number protection patterns
$number_protection_patterns = array() ;
$counts = array() ;
$fp = fopen( $file_in_number_dictionnary, 'r' ) ;
if( !$fp ) {
	trigger_error( "File not found $file_in_words_dictionnary", E_USER_ERROR ) ;
}
while( !feof( $fp ) ) {
	$line = fgets( $fp ) ;
	$cells = explode("\t", rtrim($line)) ;
	if( count($cells)>1) {
		list($pattern, $type) = $cells ;
		if( isset($number_protection_patterns[$type])) {
			$number_protection_patterns[$type][] = $pattern ;
			$counts[$type] ++ ;
		} else {
			$number_protection_patterns[$type] = array($pattern) ;
			$counts[$type] = 1 ;
		}
	}
}
fclose( $fp ) ;

print "\nThe number protection patterns contains the following elements:\n" ;
print_r($counts) ;
print "\n" ;




// ================ we treat the files


print( $bdd->get_string_lock_status($table_name) );
$i=0 ;
print "\n\n\n$i letters" ;


while( $file = $bdd->lock_file($table_name) ) {
	$a_file_in = $dir_in.'/'. $file ;
	$a_file_out = $dir_out.'/'. $file ;
	$content = file_get_contents( $a_file_in ) ;

	// First, we protect some numbers before deleting all the others.
	$number_protection_memory = array() ;

	/* We have tried to preserve fractions and drop dates, but it is too dangerous. As soon as there is a slash, we drop it.
	$content = preg_replace("|\b(\d+/\d+/\d+)\b|i", '@/@/@' ,$content ) ;
	if( preg_match_all ( "|\b(\d+/\d+)\b|i" ,$content, $matches, PREG_PATTERN_ORDER  ) ) {
		$number_protection_memory = array_merge( $number_protection_memory, $matches[0] ) ;
	} */
	
	// FIRST: "word_before" : number protection patterns with a word before the number
	foreach( $number_protection_patterns["word_before"] as $word ) {
		if( preg_match_all ( "|\b($word\s*)(\d+ ?\d*[\.,]?\d*)|i" ,$content, $matches, PREG_PATTERN_ORDER  ) ) {
			$number_protection_memory = array_merge( $number_protection_memory, $matches[0] ) ;
		}
	}
	// SECOND: "prefix_after": number protection patterns with a word after the number: a white caracter is possible but not mandatory. In addition, the following word can eventually "begin like".
	foreach( $number_protection_patterns["prefix_after"] as $word ) {
		if( preg_match_all ( "|(\d+ ?\d*[\.,]?\d*\s*$word)|i" ,$content, $matches, PREG_PATTERN_ORDER  ) ) {
			$number_protection_memory = array_merge( $number_protection_memory, $matches[0] ) ;
		}
	}
	// THIRD: "word_after": number protection patterns with a word after the number: a white caracter is possible. The following word must be entire (not "start with"...).
	foreach( $number_protection_patterns["word_after"] as $word ) {
		if( preg_match_all ( "|(\d+ ?\d*[\.,]?\d*)(\s*$word)\b|i" ,$content, $matches, PREG_PATTERN_ORDER  ) ) {
			$number_protection_memory = array_merge( $number_protection_memory, $matches[0] ) ;
		}
	}

	// now we protect the numbers: we replace them by a special string that will be ignored later
	$number_protection_memory2 = array() ;
	foreach( $number_protection_memory as $index => $mot ) {
		$index2 = 'beginofprotectednumber'.$index.'endofprotectednumber' ; // this protection pattern doesn't contain any special character. Elsewise, it would be used to split the token again.
		$number_protection_memory2[$index2] = $mot ;
		$content = str_replace($mot, $index2 , $content) ;
	}

	// We drop the words that don't belong to the list of authorized words. By construction, those "words" also include unprotected numbers.
	$keywords = preg_split('/[\p{P}\p{S}\p{Z}\p{M}\\n\\r\s]+/', $content , -1, PREG_SPLIT_NO_EMPTY );
	foreach( $keywords as $a_word ) {
		// except if the word is a protected number
		if( ! strstr($a_word, 'beginofprotectednumber') AND ! strstr($a_word, 'endofprotectednumber') ) {
			if( $a_word!='@' AND ! in_array($a_word, $words_dictionnary) ) {
				$content = preg_replace("|\b$a_word\b|iU", '@', $content )  ;
			}
		}
	}

	// then we rewrite the protected numbers
	foreach( $number_protection_memory2 as $index2 => $word ) {
		$content = str_replace($index2,  $word , $content) ;
	}

	// finally we records the new file
	file_put_contents( $a_file_out, $content ) ;
	
	$bdd->unlock_file($table_name, $file) ;
	$i++ ;
	print "\r$i letters" ;


}


print( "\n\n". $bdd->get_string_lock_status($table_name) . "\n\n" );



?>



STEP 07 DONE
Please check the error log file if necessary.
