FASDIM is an open-source program that is designed to de-identify French free-text medical letters.
It is distributed without any warranty. Please read the terms of the GNU GPL license.
It also uses the GnuWin32 Library and WV 1.0.2, under GNU GPL license. Please read the corresponding licenses.
(C) Emmanuel Chazard, 2013-11-28, check for updates at http://fasdim.org


<?php


/*

	FASDIM: Fast And Simple DeIdentification Method
	Copyright (C) 2013 Emmanuel Chazard http://emmanuel.chazard.org
	Check for updates on http://fasdim.org
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 3 of the License.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.


*/

// ============================================ definition of some classes

 
/**
* this class is used to get the parameters from the configuration file
*/
class configurator {
	protected $content = array() ;
	protected $path = "fasdim.ini" ;
	
	public function __construct() {
		$this->content = parse_ini_file($this->path, false) ;
	}
	
	public function get($field) {
		if( isset($this->content[$field])) {
			return $this->content[$field] ;			
		} else {
			trigger_error("The field $field is not defined in the configuration file. Empty value returned", E_USER_WARNING) ;
			return FALSE ;
		}		
	}
	
	public function get_all() {
		return $this->content ;
	}	
}

/**
* Progression bar in batch mode
*/
class batch_progression_bar {
	private static $compteur ;
	private $min ;
	private $max ;
	private $avancement =0;
	private $image_gauche ;
	private $image_droite ;

	/**
	 * Constructor
	 *
	 * It sets the output bufferization off.
	 *
	 * @param int $min Floor of the scale
	 * @param int $max Ceiling of the scale
	 * @param string $image_gauche Character on the left
	 * @param string $image_droite Character on the right
	 */
	public function __construct($min=0, $max=100, $image_gauche='H', $image_droite='.') {
		if(!isset(self::$compteur)) {
			self::$compteur = 0 ;
		}
		self::$compteur ++ ;
		if($min>$max) {
			$temp = $min ;
			$min = $max ;
			$max = $temp ;
		}
		$this->min = $min ;
		$this->max = $max ;
		$this->image_gauche = $image_gauche ;
		$this->image_droite = $image_droite ;
		print "\n" ;
	}
	/**
	 * Destructor
	 */
	public function __destruct() {
		$this->min  = null ;
		$this->max  = null ;
		$this->avancement  = null ;
		$this->image_gauche  = null ;
		$this->image_droite  = null ;
	}
	/**
	 * This method draws the bar on the console
	 */
	public function draw() {
		$chaine =  str_repeat( $this->image_droite, 20 ) ;
		print $chaine ;
		return true ;
	}
	/**
	 * This method updates the progression bar
	 *
	 * @param int $nombre Progression number
	 * @return boolean Returns TRUE when the image changes and FALSE elsewhere (for information).
	 */
	public function forward($nombre) {
		$nombre = max($nombre, $this->min) ;
		$nombre = min($nombre, $this->max) ;
		/* nombre compris entre 0 et 100 */
		$temp = ceil(100 * ($nombre - $this->min) / ($this->max - $this->min)) ;
		if($temp!=$this->avancement) {
			$this->avancement = $temp ;
			$temp1 = round($temp /5)  ;
			$temp2 = 20 - $temp1 ;
			print "\r" . str_repeat( $this->image_gauche, $temp1) . str_repeat( $this->image_droite , $temp2) . "  ". str_pad ( $temp, 2,'0',STR_PAD_LEFT) ." %" ;
			return true ;
		}
		return false ;
	}
}


/**
 * This class is used to manage the database, the tables and the files listed in the tables.
 * 
 * Such as PDO, must be constructed using __construct( $bdd_dsn, $bdd_user, $bdd_pwd ) ;
 */
class fasdim_db extends PDO {

	public function get_string_lock_status($table_name) {
		$stt = $this->prepare("select statut, count(*) from $table_name group by statut order by statut ;") ;
		$stt->execute() ;
		$res = $stt->fetchAll( PDO::FETCH_NUM ) ;
		$chaine = "" ;
		foreach( $res as $resultat) {
			list( $statut, $eff ) = $resultat ;
			switch($statut) {
				case 0 :	$statut2 = "not attempted" ;	break ;
				case 1 :	$statut2 = "in progress" ;		break ;
				case 2 :	$statut2 = "done" ;				break ;
			}
			$chaine .= "\tStatus [$statut2] : $eff files.\n" ;
		}
		return $chaine ;
	}

	function lock_file($table_name) {
		$stt = $this->prepare("SELECT nom FROM $table_name WHERE statut=0 ORDER BY rand() LIMIT 1 ;") ;
		$stt->execute() ;
		$res = $stt->fetch( PDO::FETCH_NUM ) ; // only one line
		if( !$res ) {
			return false ; // no file left
		}
		$nom_fichier = $res[0] ;
		$stt2 = $this->prepare("UPDATE $table_name SET statut=1 WHERE nom = :nom ;") ;
		$stt2->execute( array(":nom"=> $nom_fichier )) ;
		$count = $stt2->rowCount();
		if( $count!=1) {
			trigger_error("Warning, $count lines modified during the locking of file $nom_fichier .", E_USER_NOTICE) ;
		}
		return $nom_fichier ;
	}


	function unlock_file($table_name, $file_name) {
		$stt2 = $this->prepare("UPDATE $table_name SET statut=2 WHERE nom = :nom ;") ;
		$stt2->execute( array(":nom"=> $file_name )) ;
		$count = $stt2->rowCount();
		if( $count!=1) {
			trigger_error("Warning, $count lines modified during the unlocking of file $file_name .", E_USER_NOTICE) ;
			return false ;
		}
		return true ;
	}


	function debug(PDOStatement $statement) {
		if( $statement->errorCode() != '00000') {
			trigger_error( implode("\t", $statement->errorInfo() ), E_USER_WARNING )  ;
			return FALSE ;
		}
		return TRUE ;
	}

	function get_file_list($table_name, $status) {
		$stt = $this->prepare("SELECT nom FROM $table_name WHERE statut=$status ;") ;
		$stt->execute() ;
		$liste=array() ;
		while( $res = $stt->fetch( PDO::FETCH_NUM ) ) {
			$liste[] = $res[0] ;
		}
		return $liste ;
	}

}


// ============================================ END OF definition of some classes


// ============================================ initiate some comon jobs

$configuration = new configurator();


ini_set("display_errors",'Off');
ini_set("error_reporting",E_ALL);
ini_set("log_errors",'On');
ini_set("error_log",$configuration->get('file_log'));


print "The program will be run with the following configuration.\nIt can be modified in the appropriate *.ini file.\n" ;


foreach( $configuration->get_all() as $key => $value) {
	if( !isset($liste_param) or in_array($key, $liste_param)) {
		print str_pad($key,30)."$value\n" ;
	}	
}

print "\nContinue with those parameters (y/n)?\n" ;
$answer = strtolower(trim(fgets(STDIN))) ;

if( $answer!='y') {
	exit ;
}


// create the connexion to the database
try {
	$bdd = new fasdim_db(  
		$configuration->get('database_dsn'), 
		$configuration->get('database_user'), 
		$configuration->get('database_password')
	) ;
} catch (PDOException $e) {
	trigger_error('Connection failed: ' . $e->getMessage(), E_USER_ERROR) ;
}



?>
